@@ -164,7 +164,7 @@ find_vmm_phys (void)
 	phys_t base;
 	phys_t len;
 	u32 type;
-	int index;
+	int index = 0;
 
 	phys = 0;
 	while(get_phys_mem_map(index++, &base, &len, &type)) {
@@ -544,18 +544,24 @@ pmap_read (pmap_t *m)
 		if (!(tmp & PDE_P_BIT))
 			return 0;
 		if (m->readlevel == 1 && (tmp & PDE_PS_BIT)) {
-			tmp &= ~PDE_PS_BIT;
-			if (tmp & PDE_4M_PAT_BIT) {
-				tmp |= PTE_PAT_BIT;
-				tmp &= ~PDE_4M_PAT_BIT;
-			}
-			if (m->levels == 2) {
-				tmp |= ((tmp & 0x1E000) >> 12) << 32;
-				tmp &= ~0x3FF000ULL;
-				tmp |= (m->curaddr & 0x3FF000);
-			} else {
-				tmp &= ~0x1FF000ULL;
-				tmp |= (m->curaddr & 0x1FF000);
+			if (m->curlevel == 0) {
+				/*
+				 * If large page, and requested level
+				 * is 1 (m->curlevel == 0), return
+				 * pseudo PTE.
+				 */
+				tmp &= ~PDE_PS_BIT;
+				if (tmp & PDE_4M_PAT_BIT) {
+					tmp |= PTE_PAT_BIT;
+					tmp &= ~PDE_4M_PAT_BIT;
+				}
+				if (m->levels == 2) {
+					tmp &= ~0x3FF000ULL;
+					tmp |= (m->curaddr & 0x3FF000);
+				} else {
+					tmp &= ~0x1FF000ULL;
+					tmp |= (m->curaddr & 0x1FF000);
+				}
 			}
 			return tmp;
 		}
